use std::io;
use std::time::{Duration, Instant};

use crossterm::event;
use crossterm::event::{Event, KeyCode};
use tui::backend::Backend;
use tui::style::*;
use tui::text::*;
use tui::widgets::*;
use tui::{Frame, Terminal};

use sea_battle_backend::data::{BotDescription, BotType, PlayConfiguration};

use crate::consts::{HIGHLIGHT_COLOR, TICK_RATE};
use crate::ui_screens::utils::centered_rect_size;
use crate::ui_screens::ScreenResult;

pub struct SelectBotTypeScreen {
    state: ListState,
    curr_selection: usize,
    types: Vec<BotDescription>,
}

impl Default for SelectBotTypeScreen {
    fn default() -> Self {
        let types = PlayConfiguration::default().bot_types;
        Self {
            state: Default::default(),
            curr_selection: types.len() - 1,
            types: types.to_vec(),
        }
    }
}

impl SelectBotTypeScreen {
    pub fn show<B: Backend>(
        mut self,
        terminal: &mut Terminal<B>,
    ) -> io::Result<ScreenResult<BotType>> {
        let mut last_tick = Instant::now();
        loop {
            self.state.select(Some(self.curr_selection));
            terminal.draw(|f| self.ui(f))?;

            let timeout = TICK_RATE
                .checked_sub(last_tick.elapsed())
                .unwrap_or_else(|| Duration::from_secs(0));

            if event::poll(timeout)? {
                if let Event::Key(key) = event::read()? {
                    match key.code {
                        KeyCode::Char('q') | KeyCode::Esc => return Ok(ScreenResult::Canceled),
                        KeyCode::Enter => {
                            return Ok(ScreenResult::Ok(self.types[self.curr_selection].r#type));
                        }
                        KeyCode::Down => self.curr_selection += 1,
                        KeyCode::Up => self.curr_selection += self.types.len() - 1,
                        _ => {}
                    }

                    self.curr_selection %= self.types.len();
                }
            }
            if last_tick.elapsed() >= TICK_RATE {
                last_tick = Instant::now();
            }
        }
    }

    fn ui<B: Backend>(&mut self, f: &mut Frame<B>) {
        let area = centered_rect_size(60, self.types.len() as u16 * 2 + 2, &f.size());

        // Create a List from all list items and highlight the currently selected one
        let items = self
            .types
            .iter()
            .map(|bot| {
                ListItem::new(vec![
                    Spans::from(bot.name),
                    Spans::from(Span::styled(
                        bot.description,
                        Style::default().add_modifier(Modifier::ITALIC),
                    )),
                ])
            })
            .collect::<Vec<_>>();
        let items = List::new(items)
            .block(
                Block::default()
                    .title("Select bot type")
                    .borders(Borders::ALL),
            )
            .highlight_style(
                Style::default()
                    .fg(HIGHLIGHT_COLOR)
                    .add_modifier(Modifier::BOLD),
            )
            .highlight_symbol(">> ");

        f.render_stateful_widget(items, area, &mut self.state);
    }
}
